/***************************************************************************
 * GraphGenerator.java
 *
 * 2 novembre 2006, 16.02
 * Copyright  2006  Alberto Casagrande <alberto.casagrande@gmail.com>
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package Ladder.main;

import java.util.Vector;
import java.util.LinkedList;

/**
 *
 * @author  Alberto Casagrande <alberto.casagrande@gmail.com>
 */
public class GraphGenerator {
    
    private String[] GetM_Lengths (Object [][] table) {
        
        int columns=table[0].length;
        
        String[] m_lengths=new String[columns];
        
        for (int x=0; x< columns; x++) {
            m_lengths[x]=table[0][x].toString();
        }
        
        return m_lengths;
    }
    
    private String[] GetSpecies (Object [][] table) {
        
        int rows=table.length;
        
        String [] species=new String[rows];
        
        for (int y=0; y< rows; y++) {
            species[y]=table[y][0].toString();
        }
        
        return species;
    }
    
    private void FillM_Lengths (Object [][] table, int column) throws java.io.IOException, FormatError{
        
        if (table[0].length<column) throw  new java.io.IOException("GraphGenerator::FillM_Lengths: The" +
                "input matrix has "+ table[0].length +" columns and you require to work on column "+ column);
        
        int rows=table.length;
        int len1, len2;
        StringParser parser;
        String s; 
        
        String[] m_lengths = GetM_Lengths(table);     
        
        String[] species = GetSpecies(table);
        
        int y=0;
        
        try {
           for (y=1; y< rows; y++) {
               parser =new StringParser(table[y][column].toString());
               len1=parser.ReadNumber();
               parser.ReadChar('-');
               len2=parser.ReadNumber();
               if (!parser.NoMoreToParse()) throw new ParserError();
               AddMicroSat(species[y],m_lengths[column],len1,len2);
           }
        } catch (ParserError e) {
           throw new FormatError(y,column); 
        }
   
    }
    
    private void AddMicroSat(String specie, String microsat, int len1, int len2) 
                                                   throws java.io.IOException{
        for (int i=0; i< m_lengths.size(); i++) {
            if (((MicroSatLengths)(m_lengths.get(i))).HasSameLengths(len1,len2)) {
                ((MicroSatLengths)m_lengths.get(i)).AddMicroSat(specie);
                return;
            }
        }
        m_lengths.add(new MicroSatLengths(specie, len1, len2));
    }
    
    /** Creates a new instance of GraphGenerator */
    public GraphGenerator(Object [][] table, int column) throws java.io.IOException, FormatError {
        m_lengths= new Vector();
        FillM_Lengths(table, column);
    }
    
    public Vector Edges() {
        return m_lengths;
    }
    
    public MicroSatLengths Edge(int idx) {
        return (MicroSatLengths)m_lengths.get(idx);
    }
    
    public void print() {
        
        for (int i=0; i<m_lengths.size(); i++) {
            ((MicroSatLengths)m_lengths.get(i)).print();
        }
    }
    
    private Vector m_lengths;
}
