/***************************************************************************
 * MaxMatchCaller.java
 *
 * 3 novembre 2006, 10.57
 * Copyright  2006  Alberto Casagrande <alberto.casagrande@gmail.com>
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package Ladder.main;

import java.io.*;
import java.util.Vector;
import java.util.LinkedList;
import Ladder.main.Solution;

/**
 *
 * @author  Alberto Casagrande <alberto.casagrande@gmail.com>
 */
public class MatchFinderCaller {
    
    public MatchFinderCaller() {
        File file= new File("./matchfinder");
        if (file.isFile()) {
            matchfinder=new String("./matchfinder");
        } else {
            matchfinder=new String("matchfinder.exe");
        }     
    }
    
    public Solution FindMaxMatch(GraphGenerator g) throws IOException, ParserError {
            
        Process mf_p = Runtime.getRuntime().exec(matchfinder);
        BufferedReader input = new BufferedReader(
                    new InputStreamReader(mf_p.getInputStream()));
        BufferedWriter output = new BufferedWriter(
                    new OutputStreamWriter(mf_p.getOutputStream()));
            
        Vector edges=g.Edges();
            
        WriteEdgesInInput(output, edges);
        output.close();
            
        Vector maxmatch=ParseMatch(input);
           
        input.close();
            
        return GetMatchEdgesIndex(edges, maxmatch);
    }
    
    public Solution FindMinCover(GraphGenerator g) throws IOException, ParserError {
            
        Process mf_p = Runtime.getRuntime().exec(matchfinder+" -c");
        BufferedReader input = new BufferedReader(
                    new InputStreamReader(mf_p.getInputStream()));
        BufferedWriter output = new BufferedWriter(
                    new OutputStreamWriter(mf_p.getOutputStream()));
            
        Vector edges=g.Edges();
            
        WriteEdgesInInput(output, edges);
        output.close();
            
        Vector mincover=ParseCover(input);
           
        input.close();
            
        return GetCoverEdgesIndex(edges, mincover);
    }
    
    
    private Vector GetVerteces(Vector edges) {
        Vector lengths=new Vector();
        
        for (int j=0; j<edges.size(); j++) {
            MicroSatLengths m_lengths=(MicroSatLengths)edges.get(j);
            
            if (!lengths.contains(m_lengths.FirstLength())) {
                lengths.add(m_lengths.FirstLength());
            }
            if (!lengths.contains(m_lengths.SecondLength())) {
                lengths.add(m_lengths.SecondLength());
            }
        }
        
        return lengths;
    }
    
    private int [] GetUnmachedVerteces(Vector edges, Vector maxmatch) {
        Vector lengths=GetVerteces(edges);
       
        boolean[] matched_lengths_bool= new boolean[lengths.size()];
        int matched_lengths=lengths.size();
        
        for (int j=0; j<lengths.size(); j++) {
            int length=((Integer)(lengths.get(j)));
            matched_lengths_bool[j]=false;
            for (int i=0; i<maxmatch.size(); i++) {
                MicroSatLengths m_lengths=(MicroSatLengths)maxmatch.get(i);
                if ((m_lengths.FirstLength()==length)||
                     (m_lengths.SecondLength()==length)){
                   matched_lengths_bool[j]=true;
                   matched_lengths=matched_lengths-1;
                }
            }
        }
        
        int [] matched_lengths_vect=new int[matched_lengths];
        int i=0;
        
        for (int j=0; j<lengths.size(); j++) {
            if (!matched_lengths_bool[j]) {
                matched_lengths_vect[i]=((Integer)lengths.get(j)).intValue();
                i=i+1;
            }
        }
        
        return matched_lengths_vect;
    }

    private int[] GetSolutionEdgesIndex(Vector edges, Vector sol_edges) {
        int[] edges_idx=new int[sol_edges.size()];
        
        for (int j=0; j<edges.size(); j++) {
            MicroSatLengths m_lengths=(MicroSatLengths)edges.get(j);
            for (int i=0; i<sol_edges.size(); i++) {
                if (((MicroSatLengths)sol_edges.get(i)).
                        HasSameLengths(m_lengths)) {
                    
                    edges_idx[i]=j;
                }
            }
        }
        
        return edges_idx;
    }
        
    private Solution GetMatchEdgesIndex(Vector edges, Vector maxmatch) {
        int[] edges_idx=GetSolutionEdgesIndex(edges,maxmatch);
        int[] unmatched_lengths=GetUnmachedVerteces(edges, maxmatch);
        
        return new Solution(edges_idx,unmatched_lengths);
    }
    
    private Solution GetCoverEdgesIndex(Vector edges, Vector mincover) {
        int[] edges_idx=GetSolutionEdgesIndex(edges,mincover);
        
        return new Solution(edges_idx);
    }
    
    private void WriteEdgesInInput(BufferedWriter output, Vector edges)
                                           throws java.io.IOException {
        
        boolean first=true;
        
        output.write("E={");
        //System.out.print("E={");
        for (int i=0; i<edges.size(); i++) {
            MicroSatLengths al=(MicroSatLengths)edges.get(i);
            if (!first) {
                output.write(", ");
                //System.out.print(", ");
            } else {
                first=false;
            }
             
            output.write("("+al.FirstLength()+","+al.SecondLength()+ ")");
            //System.out.print("("+al.FirstLength()+","+al.SecondLength()+ ")");
        }
        
        output.write("}");
        //System.out.print("}");
    }
    
    private Vector ParseMatch(BufferedReader input) 
                                           throws ParserError, java.io.IOException {
        
        return ParseOutput(input,"Match");
    }
    
    private Vector ParseCover(BufferedReader input) 
                                           throws ParserError, java.io.IOException {
        
        return ParseOutput(input,"Cover");
    }
    
    private Vector ParseOutput(BufferedReader input, String s) 
                                           throws ParserError, java.io.IOException {
        String input_line;
        Vector maxmatch= new Vector();
        boolean not_f_edge=false, f_line=true;
            
        while ((input_line = input.readLine()) != null) {
           
            StringParser parser= new StringParser(input_line);
            
            if (f_line) {
                parser.ReadString(s);    
                parser.ReadChar('=');
                parser.ReadChar('{');
            }
            
            while (!parser.Done()) {
                if (not_f_edge) {
                    try {
                        parser.ReadChar(',');
                    } catch (ParserError e) {
                        parser.ReadChar('}');
                    }
                }
                parser.RemoveBlank();
                if (!parser.Done()) {
                    parser.ReadChar('(');
                    int len1=parser.ReadNumber();
                    parser.ReadChar(',');
                    int len2=parser.ReadNumber();
                    parser.ReadChar(')');
                    maxmatch.add(new MicroSatLengths(len1, len2));
                }
                not_f_edge=true;
            }

        }     
        
        return maxmatch;
    }
    
    private String matchfinder;

};
