/***************************************************************************
 * StringParser.java
 *
 * 2 novembre 2006, 18.07
 * Copyright  2006  Alberto Casagrande <alberto.casagrande@gmail.com>
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package Ladder.main;

import Ladder.main.ParserError;

/**
 *
 * @author  Alberto Casagrande <alberto.casagrande@gmail.com>
 */
public class StringParser {
    
    public StringParser(String s) {
        parsed_char=0;
        to_parse=s;
    }
    
    public int ReadNumber() throws ParserError{
        Character ch=new Character(to_parse.charAt(parsed_char));
        int number=0;
        boolean digit_found=false;

        RemoveBlank();
        
        while ((!Done())&&(ch.isDigit(to_parse.charAt(parsed_char)))) {
            number=number*10+ch.digit(to_parse.charAt(parsed_char),10);
            parsed_char++;
            digit_found=true;
        }
                
        if (digit_found==true) {
            return number;
        }
        
        throw new ParserError("StringParser::ReadNumber: "+
                            "Parse Error at character n."+ parsed_char +
                            " in string "+to_parse);
    }
    
    public void ReadChar(char c) throws ParserError{
        Character ch=new Character(to_parse.charAt(parsed_char));
        
        RemoveBlank();
        
        if ((!Done())&&(to_parse.charAt(parsed_char)==c)) {
            parsed_char++;
            return;
        } 
        
        throw new ParserError("LengthsParser::ReadChar: "+
                            "Parse Error at character n."+ parsed_char +
                            " in string "+to_parse);
    }
    
    public void ReadString(String s) throws ParserError{
        Character ch=new Character(to_parse.charAt(parsed_char));
        int char_in_s=0;
        int pos=parsed_char;
        
        RemoveBlank();
        
        while ((!Done())&&
                (to_parse.charAt(parsed_char)==s.charAt(char_in_s))) {
            parsed_char++;
            char_in_s++;
            if (char_in_s==s.length()) {
                return;
            }
        } 
        
        parsed_char=pos;
        
        throw new ParserError("LengthsParser::ReadString: "+
                            "Parse Error at character n."+ parsed_char +
                            " in string "+s+ " with respect to "+to_parse);
    }
    
    public void RemoveBlank() {
        while ((!Done())&&(to_parse.charAt(parsed_char)==' ')) {
            parsed_char++;
        }
    }
    
    public void ResetParser() {
        parsed_char=0;
    }
    
    public boolean Done() {
        return parsed_char==to_parse.length();
    }
    
    public boolean NoMoreToParse() {
        int pos=parsed_char;
        RemoveBlank();
        if (Done()) {
            parsed_char=pos;
            return true;
        }
        parsed_char=pos;
        return false;
    }
    
    private String to_parse;
    private int parsed_char;
    
}
