/***************************************************************************
 * SolutionWindow.java
 *
 * 4 novembre 2006, 20.30
 * Copyright  2006  Alberto Casagrande <alberto.casagrande@gmail.com>
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package Ladder.ui;

import Ladder.main.*;
import java.util.LinkedList;
import java.util.Vector;
import java.io.File;
import javax.swing.JTable;
import javax.swing.JScrollPane;

/**
 *
 * @author  Alberto Casagrande <alberto.casagrande@gmail.com>
 */
public class SolutionWindow extends javax.swing.JFrame {
    
    public SolutionWindow(Object [][]datatable) throws FormatError {
        Solution match,cover;
        match_tables=new Vector();
        xtables=new Vector();
        xscrollpane=new Vector();
        cover_tables=new Vector();
        
        try {                
            Ladder.main.MatchFinderCaller caller = new Ladder.main.MatchFinderCaller();
            
            for (int x=1; x< datatable[0].length; x++) {
                ggenerator= new Ladder.main.GraphGenerator(datatable,x);
                match=caller.FindMaxMatch(ggenerator);
                cover=caller.FindMinCover(ggenerator);
                String microsat_name=(String)datatable[0][x];
                match_tables.add(BuildSheet(microsat_name,match));
                cover_tables.add(BuildSheet(microsat_name,cover));
            }
        } catch (java.io.IOException e) {
            System.out.print("SolutionWindow:");
            System.out.println(e);
        } catch (ParserError e) {
            System.out.print("SolutionWindow:");
            System.out.println(e);
        }
      
        initComponents();
        jTabbedMatch = SetTables(match_tables);
        jTabbedPane1.addTab("Solution", jTabbedMatch);
        
        jTabbedCover = SetTables(cover_tables);
        jTabbedPane1.addTab("Cover", jTabbedCover);
    }
    
    private int get_max_free_choices_in(Solution solution) {
        int columns=0;
        int [] edge_idx=solution.edges();
        int rows=edge_idx.length;
        
        for (int i = 0; i<rows; i++) {
            MicroSatLengths m_lengths=ggenerator.Edge(edge_idx[i]);
            if ((m_lengths.getMicroSats().size())>columns) {
                columns=m_lengths.getMicroSats().size();
            }
        }
        
        return columns;
    }
    
    private Object [][] write_solution_tables(Solution solution, int r_inc, int c_inc) {
        
        int columns=get_max_free_choices_in(solution);
        int [] edge_idx=solution.edges();
        int rows=edge_idx.length;
        
        Object [][] solution_table=new Object [rows+r_inc][columns+c_inc];
        for (int y = 0; y<rows; y++) {
            MicroSatLengths m_lengths=ggenerator.Edge(edge_idx[y]);
            solution_table[y][0]=m_lengths.FirstLength()+" - "+
                         m_lengths.SecondLength();
            LinkedList microsats=m_lengths.getMicroSats();
            for (int x = 0; x< microsats.size(); x++) {
                solution_table[y][x+1]=(String)microsats.get(x);
            }
        }
        
        return solution_table;
    }
    
    public SolutionSheet BuildSheet(String microsat_name, Solution solution) {
        int [] missing_lengths=solution.missing_lengths();
        int sup_rows=0;
        
        if (missing_lengths.length!=0) sup_rows=2;
        
        Object [][] table=write_solution_tables(solution,sup_rows,1);
        
        if (missing_lengths.length!=0) {
            int rows=table.length;
            int columns=table[0].length;
            table[rows-1][0]=new String("Missing Lengths:");
            String missing=new String();
            if (0<missing_lengths.length) {
                missing=new String(missing+missing_lengths[0]);
                for (int i = 1; i<missing_lengths.length; i++) {
                    missing=new String(missing+", "+missing_lengths[i]);
                }
            }
            table[rows-1][1]=missing;
        }
        
        return new SolutionSheet(microsat_name, table);
    }
   
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        Save = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        Exit = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Solution");

        jMenu1.setText("File");
        Save.setText("Save");
        Save.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SaveActionPerformed(evt);
            }
        });

        jMenu1.add(Save);

        jMenu1.add(jSeparator1);

        Exit.setText("Close");
        Exit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ExitActionPerformed(evt);
            }
        });

        jMenu1.add(Exit);

        jMenuBar1.add(jMenu1);

        setJMenuBar(jMenuBar1);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jTabbedPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jTabbedPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 161, Short.MAX_VALUE)
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void SaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SaveActionPerformed
       try {
           if (!save_sheet.isValid()) {
               if (jTabbedPane1.getSelectedIndex()==0) {
                   save_sheet= new SaveSolutionSheets(match_tables);
               } else {
                   save_sheet= new SaveSolutionSheets(cover_tables);
               }
               save_sheet.setVisible(true);
           }
       } catch (Exception e) {
           if (jTabbedPane1.getSelectedIndex()==0) {
               save_sheet= new SaveSolutionSheets(match_tables);
           } else {
               save_sheet= new SaveSolutionSheets(cover_tables);
           }
           save_sheet.setVisible(true);
       }
    }//GEN-LAST:event_SaveActionPerformed

    private void ExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ExitActionPerformed
        dispose();
    }//GEN-LAST:event_ExitActionPerformed

    public javax.swing.JTabbedPane SetTables( Vector tables) {
        
        String [] title = new String [0];
        javax.swing.JTabbedPane jTabbedPane = new javax.swing.JTabbedPane();
        
        for (int x=0; x< tables.size(); x++) {
            JTable new_xtable=new JTable();
            JScrollPane new_xscrollpane = new JScrollPane();
           
            SolutionSheet sheet=(SolutionSheet)tables.get(x);
           
            Object[][] table=sheet.table();
       
            title = new String [table[0].length];
            for (int i=0; i<title.length; i++) {
                title[i]=new Integer(i+1).toString();
            }
                
            new_xtable.setModel(new javax.swing.table.DefaultTableModel(table,title));
            
            new_xscrollpane.setViewportView(new_xtable);

            jTabbedPane.addTab(sheet.name(), new_xscrollpane);
     
            xtables.add(new_xtable);
            //xscrollpane.add(new_xscrollpane);
        }
        
        return jTabbedPane;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem Exit;
    private javax.swing.JMenuItem Save;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JTabbedPane jTabbedPane1;
    // End of variables declaration//GEN-END:variables
    private Ladder.main.GraphGenerator ggenerator;
    private java.util.Vector match_tables;
    private java.util.Vector cover_tables;
    private javax.swing.JTabbedPane jTabbedMatch;
    private javax.swing.JTabbedPane jTabbedCover;
    private java.util.Vector xtables;
    private java.util.Vector xscrollpane;
    private SaveSolutionSheets save_sheet;
}
